@@ -1,7 +1,7 @@
 #ifndef __BROWSE_H
 #define __BROWSE_H
 
-#ifndef NO_CONFIGURE_MENUS
+class YSMListener;
 
 class BrowseMenu: public ObjectMenu {
 public:
@@ -10,16 +10,19 @@ public:
         YSMListener *smActionListener,
         YActionListener *wmActionListener,
         upath path,
-        YWindow *parent = 0);
+        YWindow *parent = nullptr);
     virtual ~BrowseMenu();
     virtual void updatePopup();
+
 private:
+    void loadItems();
+
     upath fPath;
-    time_t fModTime;    
+    time_t fModTime;
     YSMListener *smActionListener;
     IApp *app;
 };
 
 #endif
 
-#endif
+// vim: set sw=4 ts=4 et:
