@@ -253,7 +253,8 @@ public:
         }
     }
 
-    virtual int runProgram(const char *file, const char *const *args) {
+    virtual int runProgram(const char *file, const char *const *args,
+                           int fd = -1) {
         upath path;
         if (strchr(file, '/') == nullptr && strchr(argv0, '/') != nullptr) {
             path = upath(argv0).parent() + file;
@@ -263,7 +264,7 @@ public:
                     *(const char**)args = file;
             }
         }
-        return YApplication::runProgram(file, args);
+        return YApplication::runProgram(file, args, fd);
     }
 
     void runScript(const char *scriptName) {
@@ -368,6 +369,7 @@ public:
         else {
             static const char valgrind[] = "/usr/bin/valgrind";
             static const char catchsegv[] = "/usr/bin/catchsegv";
+            bool notify = true;
             const int size = 24;
             const char* args[size];
             int count = 0;
@@ -376,21 +378,22 @@ public:
             }
             else if (catchArg && access(catchsegv, X_OK) == 0) {
                 args[count++] = catchsegv;
+                notify = false;
             }
             args[count++] = icewmExe;
-            args[count++] = "--notify";
+            if (notify) {
+                args[count++] = "--notify";
+            }
             args[count] = nullptr;
             appendOptions(args, count, size);
             char* copy = nullptr;
-            if (wmoptions.length()) {
+            if (wmoptions.nonempty()) {
                 copy = strdup(wmoptions);
                 if (nonempty(copy)) {
                     int count = 0;
                     while (count < size && args[count])
                         count++;
-                    for (char* tok = strtok(copy, " ");
-                        tok; tok = strtok(nullptr, " "))
-                    {
+                    for (tokens tok(copy, " \t"); tok; ++tok) {
                         if (count + 1 < size) {
                             args[count++] = tok;
                             args[count] = nullptr;
@@ -402,6 +405,12 @@ public:
             if (copy) {
                 free(copy);
             }
+            if (wm_pid > 0 && notify == false && startup_phase == 0) {
+                fsleep(1.0);
+                if (kill(wm_pid, 0) == 0) {
+                    notified();
+                }
+            }
         }
     }
 
@@ -760,9 +769,9 @@ int main(int argc, char **argv) {
 
     xapp.loadEnv("env");
 
-    xapp.runIcewmbg();
-    xapp.runIcesound();
     xapp.runWM();
+    xapp.runIcesound();
+    xapp.runIcewmbg();
 
     int status = xapp.mainLoop();
 
