@@ -34,18 +34,20 @@ public:
     virtual void repaint();
     virtual void requestFocus(bool requestUserFocus);
     virtual void handleButton(const XButtonEvent& up);
+    virtual void updateToolTip();
 
-    virtual ref<YFont> getFont();
+    virtual YFont getFont();
     virtual YColor   getColor();
     virtual YSurface getSurface();
+    static void freeFont() { font = null; }
 
 private:
-    void obinit() { addStyle(wsNoExpose); setParentRelative(); }
+    void obinit() { addStyle(wsNoExpose | wsToolTipping); setParentRelative(); }
 
     osmart<DObject> fObject;
     osmart<LazyMenu> fMenu;
 
-    static ref<YFont> font;
+    static YFont font;
     static YColorName bgColor;
     static YColorName fgColor;
 };
