@@ -8,26 +8,40 @@
 #include <libgen.h>
 #include <unistd.h>
 #include <sys/types.h>
+#include <fnmatch.h>
 
 char const *ApplicationName = "strtest";
 static const char source[] = __FILE__;
 
-#define equal(p, s)     (0 == strcmp(cstring(p), cstring(s)))
+#define equal(p, s)     (mstring(p) == mstring(s))
 
 #define expect(u, s)    if (++testsrun, (u) == mstring(s) && equal(u, s)) \
-        ++passed; else test_failed(cstring(u), cstring(s), __LINE__)
+        ++passed; else test_failed(u, s, __LINE__)
+
+// XXX: those argument ordering and purpose := strange.
+// Also name collision with regular assert macro.
 
 #define assert(u, b)    if (++testsrun, (b)) ++passed; else \
-        test_failed(cstring(u), #b, __LINE__)
+        test_failed(mstring(u), #b, __LINE__)
 
 #define ispath(u, s)    if (++testsrun, (u) == upath(s) && equal(u, s)) \
-        ++passed; else test_failed(cstring(u), cstring(s), __LINE__)
+        ++passed; else test_failed(u, s, __LINE__)
 
 #define sequal(u, s)    if (++testsrun, equal(u, s)) \
-        ++passed; else test_failed(cstring(u), cstring(s), __LINE__)
+        ++passed; else test_failed(u, s, __LINE__)
+
+#define ASSERT_EQ(l,r) if(++testsrun, (l) == (r)) ++passed; \
+        else { test_failed(#l, #r, __LINE__); return; }
+#define EXPECT_EQ(l,r) if(++testsrun, (l) == (r)) ++passed; \
+        else test_failed(#l, #r, __LINE__);
 
 static int testsrun, passed, failed;
 static const char *prog;
+static int total_failed = 0;
+
+// basic detection of build daemon environment, where $HOME, $USER and
+// ~ and ~<user> are not necessarily related.
+bool isFakerootActive() { auto e=getenv("FAKED_MODE"); return e && *e; }
 
 class strtest {
     const char *name;
@@ -47,22 +61,19 @@ public:
     }
 };
 
-static void test_failed(cstring u, const char *s, int l)
+static void test_failed(mstring u, const char *s, int l)
 {
     printf("%s: Test failed in %s:%d: u = \"%s\", s = \"%s\"\n",
             prog, source, l,
             u == null ? "NULL" : u.c_str(),
-            s == 0 ? "NULL" : s);
+            s == nullptr ? "NULL" : s);
     ++failed;
+    ++total_failed;
 }
 
-static void test_failed(cstring u, cstring s, int l)
+static void test_failed(mstring u, mstring s, int l)
 {
-    printf("%s: Test failed in %s:%d: u = \"%s\", s = \"%s\"\n",
-            prog, source, l,
-            u == null ? "NULL" : u.c_str(),
-            s == null ? "NULL" : s.c_str());
-    ++failed;
+    return test_failed(u, s == null ? "NULL" : s.c_str(), l);
 }
 
 static void test_mstring()
@@ -94,7 +105,7 @@ static void test_mstring()
     expect(m, "");
     assert(m, m.length() == 0);
 
-    mstring n(0, (size_t) 0);
+    mstring n(nullptr, (size_t) 0);
     expect(n, "");
     assert(n, n.length() == 0);
 
@@ -210,31 +221,31 @@ static void test_mstring()
     q = ul.upper();
     expect(q, "ABCD.");
 
-    mstring u = NULL;
+    mstring u = nullptr;
     expect(u, "");
-    u = mstring((char *) NULL) + "aha";
+    u = mstring(nullptr) + "aha";
     expect(u, "aha");
-    u = mstring("aha") + NULL;
+    u = mstring("aha") + nullptr;
     expect(u, "aha");
 
     u = mstring("ab", "cd");
     expect(u, "abcd");
-    u = mstring("ab", (char *) NULL);
+    u = mstring("ab", nullptr);
     expect(u, "ab");
-    u = mstring(NULL, "cd");
+    u = mstring(nullptr, "cd");
     expect(u, "cd");
-    u = mstring((char *) NULL, (char *) NULL);
-    assert(u, u == null);
+    u = mstring(nullptr, nullptr);
+    assert(u, u.isEmpty());
 
     u = mstring("ab", "cd", "ef");
     expect(u, "abcdef");
-    u = mstring("ab", "cd", (char *) NULL);
+    u = mstring("ab", "cd", nullptr);
     expect(u, "abcd");
-    u = mstring("ab", (char *) NULL, "ef");
+    u = mstring("ab", nullptr, "ef");
     expect(u, "abef");
-    u = mstring(NULL, "cd", "ef");
+    u = mstring(nullptr, "cd", "ef");
     expect(u, "cdef");
-    u = mstring((char *) NULL, (char *) NULL, (char *) NULL);
+    u = mstring(nullptr, nullptr, nullptr);
     expect(u, "");
 
     u = mstring("#ffff").match("#fffff");
@@ -309,7 +320,7 @@ static void test_upath()
     upath eps = ep + s;
     expect(eps, "/etc/passwd/");
     upath neps = eps.name();
-    expect(neps, "");
+    assert(neps, neps.length() == 0);
     upath peps = eps.parent();
     expect(peps, "/etc");
 
@@ -342,15 +353,20 @@ static void test_upath()
     expect(a.getExtension(), ".q");
     a = "/stu/.vw";
     expect(a.getExtension(), null);
-
-    upath hm("~");
-    expect(hm.expand(), getenv("HOME"));
-    hm = "~/";
-    assert(strlen(hm.expand()), 1 + strlen(getenv("HOME")));
-    hm = "$HOME";
+    if (!isFakerootActive()) {
+        upath hm("~");
+        expect(hm.expand(), getenv("HOME"));
+        hm = "~/";
+        assert(strlen(hm.expand()), 1 + strlen(getenv("HOME")));
+    }
+    upath hm = "$HOME";
     expect(hm.expand(), getenv("HOME"));
     hm = "$HOME/";
     assert(strlen(hm.expand()), 1 + strlen(getenv("HOME")));
+    upath nothing("/else/matters");
+    EXPECT_EQ(0, nothing.fnMatch("/else/ma*"));
+    EXPECT_EQ(0, nothing.fnMatch("/el*/ma*"));
+    EXPECT_EQ(FNM_NOMATCH, nothing.fnMatch("/else/mata*"));
 }
 
 static void test_strlc()
@@ -424,8 +440,9 @@ static void test_cdir()
     strtest tester("cdir");
 
     {
-        cdir c("/etc");
-        assert(c.path(), c.isOpen());
+        const char cpath[] = "/etc";
+        cdir c(cpath);
+        assert(cpath, c.isOpen());
         int n = 0, p = 0, g = 0, r = 0;
         while (c.next()) {
             ++n;
@@ -433,21 +450,21 @@ static void test_cdir()
             if (0 == strcmp(c.entry(), "group")) ++g;
             if (0 == strcmp(c.entry(), "resolv.conf")) ++r;
         }
-        assert(c.path(), n >= 3);
-        assert(c.path(), p == 1);
-        assert(c.path(), g == 1);
-        assert(c.path(), r == 1);
+        assert(cpath, n >= 3);
+        assert(cpath, p == 1);
+        assert(cpath, g == 1);
+        assert(cpath, r == 1);
 
-        c.open();
+        c.open(cpath);
         n = 0;
         while (c.nextExt(".conf")) {
             ++n;
             const char *p = c.entry();
-            assert(c.path(),
+            assert(cpath,
                     strlen(p) >= 5 &&
                     0 == strcmp(p + strlen(p) - 5, ".conf"));
         }
-        assert(c.path(), n > 0);
+        assert(cpath, n > 0);
     }
 }
 
@@ -455,8 +472,9 @@ static void test_udir()
 {
     strtest tester("udir");
     {
-        udir u("/etc");
-        assert(u.path(), u.isOpen());
+        const char cpath[] = "/etc";
+        udir u(cpath);
+        assert(cpath, u.isOpen());
         int n = 0, p = 0, g = 0, r = 0;
         while (u.next()) {
             ++n;
@@ -464,18 +482,18 @@ static void test_udir()
             if (u.entry() == "group") ++g;
             if (u.entry() == "resolv.conf") ++r;
         }
-        assert(u.path(), n >= 3);
-        assert(u.path(), p == 1);
-        assert(u.path(), g == 1);
-        assert(u.path(), r == 1);
+        assert(cpath, n >= 3);
+        assert(cpath, p == 1);
+        assert(cpath, g == 1);
+        assert(cpath, r == 1);
 
-        u.open("/etc");
+        u.open(cpath);
         n = 0;
         while (u.nextExt(".conf")) {
             ++n;
-            assert(u.path(), u.entry().endsWith(".conf"));
+            assert(cpath, u.entry().endsWith(".conf"));
         }
-        assert(u.path(), n > 0);
+        assert(cpath, n > 0);
     }
 }
 
@@ -484,8 +502,9 @@ static void test_adir()
     strtest tester("adir");
 
     {
-        adir a("/etc");
-        assert(a.path(), a.isOpen());
+        const char cpath[] = "/etc";
+        adir a(cpath);
+        assert(cpath, a.isOpen());
         char buf[300] = "";
         while (a.next()) {
             const char *e = a.entry();
@@ -502,11 +521,12 @@ static void test_sdir()
     strtest tester("sdir");
 
     {
-        sdir s("/etc");
-        assert(s.path(), s.isOpen());
+        const char cpath[] = "/etc";
+        sdir s(cpath);
+        assert(cpath, s.isOpen());
         char buf[300] = "";
         while (s.next()) {
-            cstring c(s.entry());
+            mstring c(s.entry());
             const char *e = c.c_str();
             assert(e, strcoll(buf, e) < 0);
             strlcpy(buf, e, sizeof buf);
@@ -516,11 +536,89 @@ static void test_sdir()
 
 }
 
+static void test_expand()
+{
+    strtest tester("expand");
+
+    char* home = getenv("HOME");
+    if (home) {
+        char* tilde = tilde_expansion("~/");
+        assert("~/", tilde != nullptr);
+        if (tilde) {
+            assert("~/", strncmp(tilde, home, strlen(home)) == 0);
+            assert("~/", strlen(tilde) == strlen(home) + 1);
+            delete[] tilde;
+        }
+    }
+
+    if (!isFakerootActive()) {
+        char* user = getenv("USER");
+        if (user) {
+            char buf[1024];
+            snprintf(buf, sizeof buf, "~%s/", user);
+            char* uhome = tilde_expansion(buf);
+            assert(buf, uhome != nullptr);
+            if (uhome) {
+                assert(buf, strncmp(uhome, home, strlen(home)) == 0);
+                assert(buf, strlen(uhome) == strlen(home) + 1);
+                delete[] uhome;
+            }
+        }
+
+        char rhome[] = "/root";
+        if (access(rhome, 0) == 0) {
+            char* tilde = tilde_expansion("~root/");
+            assert("~root/", tilde != nullptr);
+            if (tilde) {
+                assert("~root/", strcmp(tilde, "/root/") == 0);
+                delete[] tilde;
+            }
+        }
+    }
+    if (access("/usr/bin/printf", X_OK) == 0 &&
+        access("/bin/printf", X_OK) != 0)
+    {
+        char* print = path_lookup("printf");
+        assert("printf", print != nullptr);
+        if (print) {
+            assert("printf", strcmp(print, "/usr/bin/printf") == 0);
+            delete[] print;
+        }
+    }
+
+    home = getenv("HOME");
+    if (home) {
+        char dhome[] = "$HOME/";
+        char buf[4096];
+        snprintf(buf, sizeof buf, "%s/", home);
+        char* expand = dollar_expansion(dhome);
+        assert(dhome, expand != nullptr);
+        if (expand) {
+            assert(dhome, strcmp(expand, buf) == 0);
+            delete[] expand;
+        }
+    }
+
+    auto user = getenv("USER");
+    if (user) {
+        char duser[] = "${USER}/";
+        char buf[4096];
+        snprintf(buf, sizeof buf, "%s/", user);
+        char* expand = dollar_expansion(duser);
+        assert(duser, expand != nullptr);
+        if (expand) {
+            assert(duser, strcmp(expand, buf) == 0);
+            delete[] expand;
+        }
+    }
+}
+
 int main(int argc, char **argv)
 {
     prog = basename(argv[0]);
 
     test_mstring();
+    test_expand();
     test_upath();
     test_strlc();
     test_cdir();
@@ -528,7 +626,7 @@ int main(int argc, char **argv)
     test_adir();
     test_sdir();
 
-    return 0;
+    return total_failed != 0;
 }
 
 // vim: set sw=4 ts=4 et:
