@@ -23,6 +23,7 @@
 #include "WinMgr.h"
 #define GUI_EVENT_NAMES
 #include "guievent.h"
+#include "logevent.h"
 
 /// _SET would be nice to have
 #define _NET_WM_STATE_REMOVE 0
@@ -37,8 +38,6 @@
 #define _NET_WM_BOTTOMLEFT  3
 
 #define KEY_MODMASK(x) ((x) & (ControlMask | ShiftMask | Mod1Mask))
-#define BUTTON_MASK(x) ((x) & (Button1Mask | Button2Mask | Button3Mask))
-#define BUTTON_MODMASK(x) ((x) & (ControlMask | ShiftMask | Mod1Mask | Button1Mask | Button2Mask | Button3Mask))
 
 #define COUNT(a)    (int(sizeof a / sizeof(*a)))
 #define CTRL(k)     ((k) & 0x1F)
@@ -52,6 +51,7 @@ static struct Layout {
 } layout = { 0L, 4L, 3L, _NET_WM_TOPLEFT, };
 
 typedef unsigned long Pixel;
+const char* ApplicationName = "testnetwmhints";
 
 static void tell(const char* msg, ...) {
     FILE* output = stdout;
@@ -89,6 +89,7 @@ static Colormap colormap;
 static Window root = None;
 static Window window = None;
 static Window unmapped = None;
+static Window unviewed = None;
 ///static GC gc;
 
 static long workspaceCount = 4;
@@ -100,7 +101,9 @@ static long windowWorkspace = 0;
 ///static bool fullscreen = true;
 
 class TAtom {
-    const char* name;
+public:
+    const char* const name;
+private:
     Atom atom;
 public:
     explicit TAtom(const char* name) : name(name), atom(None) { }
@@ -108,14 +111,18 @@ public:
         atom = XInternAtom(display, name, False); }
 };
 
-static TAtom _XA_ICEWM_GUI_EVENT("ICEWM_GUI_EVENT");
+static TAtom _XA_WM_CLIENT_LEADER("WM_CLIENT_LEADER");
+static TAtom _XA_WM_DELETE_WINDOW("WM_DELETE_WINDOW");
+static TAtom _XA_WM_PROTOCOLS("WM_PROTOCOLS");
+static TAtom _XA_WM_TAKE_FOCUS("WM_TAKE_FOCUS");
 static TAtom _XA_WM_STATE("WM_STATE");
 static TAtom _XA_WIN_TRAY("WIN_TRAY");
-static TAtom _XA_WIN_WORKSPACE("_WIN_WORKSPACE");
-static TAtom _XA_WIN_WORKSPACE_COUNT("_WIN_WORKSPACE_COUNT");
-static TAtom _XA_WIN_WORKSPACE_NAMES("_WIN_WORKSPACE_NAMES");
-static TAtom _XA_WIN_STATE("_WIN_STATE");
 static TAtom _XA_WIN_LAYER("_WIN_LAYER");
+static TAtom _XA_ICEWM_GUI_EVENT("ICEWM_GUI_EVENT");
+static TAtom _XA_NET_WM_DESKTOP("_NET_WM_DESKTOP");
+static TAtom _XA_NET_CURRENT_DESKTOP("_NET_CURRENT_DESKTOP");
+static TAtom _XA_NET_NUMBER_OF_DESKTOPS("_NET_NUMBER_OF_DESKTOPS");
+static TAtom _XA_NET_DESKTOP_NAMES("_NET_DESKTOP_NAMES");
 static TAtom _XA_NET_WM_ACTION_ABOVE("_NET_WM_ACTION_ABOVE");
 static TAtom _XA_NET_WM_ACTION_BELOW("_NET_WM_ACTION_BELOW");
 static TAtom _XA_NET_WM_ACTION_CHANGE_DESKTOP("_NET_WM_ACTION_CHANGE_DESKTOP");
@@ -147,12 +154,10 @@ static TAtom _XA_NET_WM_STATE_STICKY("_NET_WM_STATE_STICKY");
 static TAtom _XA_NET_WM_MOVERESIZE("_NET_WM_MOVERESIZE");
 static TAtom _XA_NET_WM_PID("_NET_WM_PID");
 static TAtom _XA_NET_WM_PING("_NET_WM_PING");
+static TAtom _XA_NET_ACTIVE_WINDOW("_NET_ACTIVE_WINDOW");
 static TAtom _XA_NET_DESKTOP_LAYOUT("_NET_DESKTOP_LAYOUT");
 static TAtom _XA_NET_REQUEST_FRAME_EXTENTS("_NET_REQUEST_FRAME_EXTENTS");
 static TAtom _XA_NET_FRAME_EXTENTS("_NET_FRAME_EXTENTS");
-static TAtom _XA_WM_DELETE_WINDOW("WM_DELETE_WINDOW");
-static TAtom _XA_WM_PROTOCOLS("WM_PROTOCOLS");
-static TAtom _XA_WM_TAKE_FOCUS("WM_TAKE_FOCUS");
 static TAtom _XA_NET_WM_WINDOW_TYPE("_NET_WM_WINDOW_TYPE");
 static TAtom _XA_NET_WM_WINDOW_TYPE_COMBO("_NET_WM_WINDOW_TYPE_COMBO");
 static TAtom _XA_NET_WM_WINDOW_TYPE_DESKTOP("_NET_WM_WINDOW_TYPE_DESKTOP");
@@ -185,11 +190,12 @@ void changeWorkspace(Window w, long workspace) {
 
     xev.type = ClientMessage;
     xev.window = w;
-    xev.message_type = _XA_WIN_WORKSPACE;
+    xev.message_type = _XA_NET_WM_DESKTOP;
     xev.format = 32;
     xev.data.l[0] = workspace;
     xev.data.l[1] = CurrentTime; //xev.data.l[1] = timeStamp;
     XSendEvent(display, root, False, SubstructureNotifyMask, (XEvent *) &xev);
+    tell("changeWorkspace %ld\n", workspace);
 }
 
 void moveResize(Window w, int x, int y, int what) {
@@ -205,10 +211,10 @@ void moveResize(Window w, int x, int y, int what) {
     XSendEvent(display, root, False, SubstructureNotifyMask, (XEvent *) &xev);
 }
 
-void toggleState(Window w, Atom toggle_state) {
+void toggleState(Window w, TAtom& toggle_state) {
     XClientMessageEvent xev = {};
 
-    puts("toggle state");
+    tell("toggle state %s\n", toggle_state.name);
 
     xev.type = ClientMessage;
     xev.window = w;
@@ -222,6 +228,15 @@ void toggleState(Window w, Atom toggle_state) {
     XSendEvent(display, root, False, SubstructureNotifyMask, (XEvent *) &xev);
 }
 
+void toggleUrgency(Display* d, Window w) {
+    XWMHints* h = XGetWMHints(d, w);
+    if (!h)
+        h = XAllocWMHints();
+    h->flags = XUrgencyHint;
+    XSetWMHints(d, w, h);
+    XFree(h);
+}
+
 void setLayer(Window w, long layer) {
     XClientMessageEvent xev = {};
 
@@ -234,13 +249,15 @@ void setLayer(Window w, long layer) {
     XSendEvent(display, root, False, SubstructureNotifyMask, (XEvent *) &xev);
 }
 
+void setProperty(Window w, Atom prop, Atom type, Atom* data, size_t count) {
+    XChangeProperty(display, w, prop, type, 32, PropModeReplace,
+                    (unsigned char *) data, int(count));
+}
+
 void setLayout(Window w) {
     tell("setLayout %ld, %ld, %ld, %ld\n",
             layout.orient, layout.columns, layout.rows, layout.corner);
-    XChangeProperty(display, w,
-                    _XA_NET_DESKTOP_LAYOUT, XA_CARDINAL,
-                    32, PropModeReplace,
-                    (unsigned char *) &layout, 4);
+    setProperty(w, _XA_NET_DESKTOP_LAYOUT, XA_CARDINAL, (Atom*)&layout, 4);
 }
 
 #if 0
@@ -268,6 +285,40 @@ void requestExtents(Window w) {
     XSendEvent(display, root, False, SubstructureNotifyMask, (XEvent *) &xev);
 }
 
+static bool getProperty(Window w, Atom p, Atom t, long* c, long limit = 1L) {
+    Atom r_type;
+    int r_format;
+    unsigned long count;
+    unsigned long bytes_remain;
+    unsigned char* prop = nullptr;
+
+    if (XGetWindowProperty(display, w, p,
+                           0L, limit, False, t,
+                           &r_type, &r_format,
+                           &count, &bytes_remain,
+                           &prop) == Success && prop)
+    {
+        TEST(r_type == t && r_format == 32 && count == 1);
+        *c = ((long *)prop)[0];
+        XFree(prop);
+        return true;
+    } else {
+        return false;
+    }
+}
+
+static void updateWorkspaceCount() {
+    getProperty(root, _XA_NET_NUMBER_OF_DESKTOPS, XA_CARDINAL, &workspaceCount);
+}
+
+static void updateActiveWorkspace() {
+    getProperty(root, _XA_NET_CURRENT_DESKTOP, XA_CARDINAL, &activeWorkspace);
+}
+
+static void updateWindowWorkspace() {
+    getProperty(window, _XA_NET_WM_DESKTOP, XA_CARDINAL, &windowWorkspace);
+}
+
 static int xfail(Display* display, XErrorEvent* xev) {
 
     char message[80], req[80], number[80];
@@ -290,45 +341,49 @@ static int xfail(Display* display, XErrorEvent* xev) {
 
 static void sigcatch(int signo) {
     tell("Received signal %d: \"%s\".\n", signo, strsignal(signo));
+    if (signo == SIGALRM) {
+        Display* d = XOpenDisplay(nullptr);
+        toggleUrgency(d, window);
+        XCloseDisplay(d);
+    }
 }
 
-int main(int argc, char **argv) {
+static void help(char* name) {
+    printf("Usage: %s [options]\n"
+            "\t-d display\n"
+            "\t-e : handle errors\n"
+            "\t-p : enable pinging\n"
+            "\t-s : synchronize\n"
+            , name);
+    exit(1);
+}
 
-    signal(SIGTERM, sigcatch);
-
-    bool pinging = false;
-
-    for (int opt, ok = 1; ok && (opt = getopt(argc, argv, "d:eps")) > 0; ) {
-        switch (opt) {
-        case 'd':
-            setenv("DISPLAY", optarg, True);
-            break;
-
-        case 'e':
-            XSetErrorHandler(xfail);
-            break;
-
-        case 'p':
-            pinging ^= true;
-            break;
-
-        case 's':
-            XSynchronize(display, True);
-            break;
-
-        default:
-            ok = 0;
-        }
-    }
-    if (optind < argc) {
-        fprintf(stderr, "%s: bad arg '%s'.\n", *argv, argv[optind]);
-        exit(1);
-    }
+static Pixel getColor(const char* name) {
+    int screen = XDefaultScreen(display);
+    colormap = XDefaultColormap(display, screen);
+    XColor color = {}, exact;
+    Pixel pixel;
+    if (XLookupColor(display, colormap, name, &exact, &color) &&
+        XAllocColor(display, colormap, &color))
+        pixel = color.pixel;
+    else if (XParseColor(display, colormap, name, &color) &&
+        XAllocColor(display, colormap, &color))
+        pixel = color.pixel;
+    else
+        pixel = XWhitePixel(display, screen);
+    tell("color %s = 0x%lX\n", name, pixel);
+    return pixel;
+}
 
+static void test_run(char* progname, bool pinging) {
     int screen = XDefaultScreen(display);
     root = XRootWindow(display, screen);
     colormap = XDefaultColormap(display, screen);
-    Pixel black = XBlackPixel(display, screen);
+    // Pixel black = XBlackPixel(display, screen);
+    Pixel white = XWhitePixel(display, screen);
+    Pixel blue = getColor("blue");
+    Pixel green = getColor("green");
+    Pixel yellow = getColor("yellow");
 
     window = XCreateWindow(display, root,
                            0,
@@ -338,7 +393,7 @@ int main(int argc, char **argv) {
                            CopyFromParent, InputOutput, CopyFromParent,
                            None, None);
 
-    XSetWindowBackground(display, window, black);
+    XSetWindowBackground(display, window, blue);
 
     Atom protocols[] = {
         _XA_WM_DELETE_WINDOW, _XA_WM_TAKE_FOCUS,
@@ -346,9 +401,9 @@ int main(int argc, char **argv) {
     };
     XSetWMProtocols(display, window, protocols, COUNT(protocols) );
 
-    XClassHint classHint = { (char *)"name:1", (char *)"class 1" };
+    XClassHint classHint = { (char *)"window", (char *)"testnet" };
     XSetClassHint(display, window, &classHint);
-    XStoreName(display, window, basename(*argv));
+    XStoreName(display, window, basename(progname));
 
     char hostname[HOST_NAME_MAX + 1] = {};
     gethostname(hostname, HOST_NAME_MAX);
@@ -361,10 +416,7 @@ int main(int argc, char **argv) {
     XSetWMClientMachine(display, window, &hname);
 
     XID pid = getpid();
-    XChangeProperty(display, window,
-                    _XA_NET_WM_PID, XA_CARDINAL,
-                    32, PropModeReplace,
-                    (unsigned char *)&pid, 1);
+    setProperty(window, _XA_NET_WM_PID, XA_CARDINAL, &pid, 1);
 
     long imask = ExposureMask | StructureNotifyMask |
                  ButtonPressMask | ButtonReleaseMask |
@@ -374,36 +426,52 @@ int main(int argc, char **argv) {
 
     XSelectInput(display, root, PropertyChangeMask);
 
+    Window leader = XCreateWindow(display, root, 0, 0, 10, 10, 0, CopyFromParent,
+                                  InputOnly, CopyFromParent, None, None);
+    setProperty(window, _XA_WM_CLIENT_LEADER, XA_WINDOW, &leader, 1);
+
     XMapRaised(display, window);
 
-    unmapped = XCreateSimpleWindow(display, root, 0, 0, 100, 100, 20, 0, black);
-    XClassHint unclassHint = { (char *)"unmapped:1", (char *)"class 1" };
+    unmapped = XCreateSimpleWindow(display, root, 0, 0, 100, 100, 20,
+                                   white, yellow);
+    XClassHint unclassHint = { (char *)"unmapped", (char *)"testnet" };
     XSetClassHint(display, unmapped, &unclassHint);
     XStoreName(display, unmapped, "unmapped");
+    setProperty(unmapped, _XA_NET_WM_PID, XA_CARDINAL, &pid, 1);
+
+    setProperty(unmapped, _XA_WM_CLIENT_LEADER, XA_WINDOW, &leader, 1);
+
+    unviewed = XCreateSimpleWindow(display, root, 0, 0, 100, 100, 20,
+                                   white, green);
+    XClassHint vwclassHint = { (char *)"unviewed", (char *)"testnet" };
+    XSetClassHint(display, unviewed, &vwclassHint);
+    XStoreName(display, unviewed, "unviewed");
+    setProperty(unviewed, _XA_NET_WM_PID, XA_CARDINAL, &pid, 1);
+
+    setProperty(unviewed, _XA_WM_CLIENT_LEADER, XA_WINDOW, &leader, 1);
 
     Atom wmstate[] = {
         // _XA_NET_WM_STATE_ABOVE,
         // _XA_NET_WM_STATE_BELOW,
-        // _XA_NET_WM_STATE_DEMANDS_ATTENTION,
+        _XA_NET_WM_STATE_DEMANDS_ATTENTION,
         // _XA_NET_WM_STATE_FOCUSED,
         // _XA_NET_WM_STATE_FULLSCREEN,
         // _XA_NET_WM_STATE_HIDDEN,
         // _XA_NET_WM_STATE_MAXIMIZED_HORZ,
         // _XA_NET_WM_STATE_MAXIMIZED_VERT,
-        // _XA_NET_WM_STATE_MODAL,
+        _XA_NET_WM_STATE_MODAL,
         // _XA_NET_WM_STATE_SHADED,
         // _XA_NET_WM_STATE_SKIP_PAGER,
         // _XA_NET_WM_STATE_SKIP_TASKBAR,
-        // _XA_NET_WM_STATE_STICKY,
+        _XA_NET_WM_STATE_STICKY,
     };
-    XChangeProperty(display, unmapped,
-                    _XA_NET_WM_STATE, XA_ATOM,
-                    32, PropModeReplace,
-                    (unsigned char *) wmstate, COUNT(wmstate));
+    setProperty(unmapped, _XA_NET_WM_STATE, XA_ATOM, wmstate, COUNT(wmstate));
+    setProperty(unviewed, _XA_NET_WM_STATE, XA_ATOM, wmstate, COUNT(wmstate));
+
     Atom wintype[] = {
         // _XA_NET_WM_WINDOW_TYPE_COMBO,
         // _XA_NET_WM_WINDOW_TYPE_DESKTOP,
-        // _XA_NET_WM_WINDOW_TYPE_DIALOG,
+        _XA_NET_WM_WINDOW_TYPE_DIALOG,
         // _XA_NET_WM_WINDOW_TYPE_DND,
         // _XA_NET_WM_WINDOW_TYPE_DOCK,
         // _XA_NET_WM_WINDOW_TYPE_DROPDOWN_MENU,
@@ -413,13 +481,14 @@ int main(int argc, char **argv) {
         // _XA_NET_WM_WINDOW_TYPE_POPUP_MENU,
         // _XA_NET_WM_WINDOW_TYPE_SPLASH,
         // _XA_NET_WM_WINDOW_TYPE_TOOLBAR,
-        _XA_NET_WM_WINDOW_TYPE_TOOLTIP,
+        // _XA_NET_WM_WINDOW_TYPE_TOOLTIP,
         // _XA_NET_WM_WINDOW_TYPE_UTILITY,
     };
-    XChangeProperty(display, unmapped,
-                    _XA_NET_WM_WINDOW_TYPE, XA_ATOM,
-                    32, PropModeReplace,
-                    (unsigned char *) wintype, COUNT(wintype));
+    setProperty(unmapped, _XA_NET_WM_WINDOW_TYPE, XA_ATOM,
+                wintype, COUNT(wintype));
+    setProperty(unviewed, _XA_NET_WM_WINDOW_TYPE, XA_ATOM,
+                wintype, COUNT(wintype));
+
     Atom actions[] = {
         _XA_NET_WM_ACTION_ABOVE,
         _XA_NET_WM_ACTION_BELOW,
@@ -435,12 +504,16 @@ int main(int argc, char **argv) {
         _XA_NET_WM_ACTION_SHADE,
         _XA_NET_WM_ACTION_STICK,
     };
-    XChangeProperty(display, unmapped,
-                    _XA_NET_WM_ALLOWED_ACTIONS, XA_ATOM,
-                    32, PropModeReplace,
-                    (unsigned char *) actions, COUNT(actions));
+    setProperty(unmapped, _XA_NET_WM_ALLOWED_ACTIONS, XA_ATOM,
+                actions, COUNT(actions));
+    setProperty(unviewed, _XA_NET_WM_ALLOWED_ACTIONS, XA_ATOM,
+                actions, COUNT(actions));
 
     XSelectInput(display, unmapped, imask);
+    XSelectInput(display, unviewed, imask);
+
+    updateWorkspaceCount();
+    updateActiveWorkspace();
 
     XEvent xev = {};
 /// XButtonEvent &button = xev.xbutton;
@@ -451,7 +524,7 @@ int main(int argc, char **argv) {
     while (XNextEvent(display, &xev) == Success) switch (xev.type) {
 
     case ClientMessage: {
-        if (client.message_type == _XA_WM_PROTOCOLS) {
+        if (client.message_type == _XA_WM_PROTOCOLS && client.window == window) {
             TEST(client.format == 32);
             if (client.data.l[0] == _XA_WM_DELETE_WINDOW) {
                 tell("WM_DELETE_WINDOW\n");
@@ -460,6 +533,7 @@ int main(int argc, char **argv) {
             else
             if (client.data.l[0] == _XA_WM_TAKE_FOCUS) {
                 tell("WM_TAKE_FOCUS\n");
+                XSetInputFocus(display, window, None, CurrentTime);
                 break;
             }
             else
@@ -494,7 +568,43 @@ int main(int argc, char **argv) {
             k = CTRL(k);
 
         if (k == 'q' || k == XK_Escape)
-            return 0;
+            return;
+        else if (k == '?' || k == 'h') {
+            printf("0 : layer desktop\n"
+                   "1 : layer below\n"
+                   "2 : layer normal\n"
+                   "3 : layer on top\n"
+                   "4 : layer dock\n"
+                   "5 : layer above dock\n"
+                   "? : help\n"
+                   "A : urgent alarm 5 seconds\n"
+                   "M : toggle state modal\n"
+                   "a : toggle state above\n"
+                   "b : toggle state below\n"
+                   "d : toggle state urgent\n"
+                   "f : toggle fullscreen\n"
+                   "s : toggle sticky\n"
+                   "t : toggle skip taskbar\n"
+                   "m : move resize 8\n"
+                   "r : move resize 4\n"
+                   "u : map the unmapped\n"
+                   "v : map the unviewed\n"
+                   "x : extents unmapped\n"
+                   "X : extents window\n"
+                   "^X : extents root\n"
+                   "Left : goto previous workspace\n"
+                   "Right : goto next workspace\n"
+                   "Shift+Left : move to previous workspace\n"
+                   "Shift+Right : move to next workspace\n"
+                   "^B : layout bottom left\n"
+                   "^C : control layout dimensions\n"
+                   "^H : horizontal layout orientation\n"
+                   "^L : layout top left\n"
+                   "^R : layout bottom right\n"
+                   "^T : layout top right\n"
+                   "^V : vertical layout orientation\n"
+                   "\n");
+        }
         else if (k == XK_Left && m == 0)
             changeWorkspace(root,
                             (workspaceCount +
@@ -515,16 +625,17 @@ int main(int argc, char **argv) {
             toggleState(window, _XA_NET_WM_STATE_ABOVE);
         else if (k == 'b')
             toggleState(window, _XA_NET_WM_STATE_BELOW);
+        else if (k == 'd')
+            toggleState(window, _XA_NET_WM_STATE_DEMANDS_ATTENTION);
         else if (k == 'M')
             toggleState(window, _XA_NET_WM_STATE_MODAL);
+        else if (k == 's')
+            toggleState(window, _XA_NET_WM_STATE_STICKY);
         else if (k == 't')
             toggleState(window, _XA_NET_WM_STATE_SKIP_TASKBAR);
-#if 0
-            //toggleState(window, WinStateAllWorkspaces);
-        /*
-        else if (k == 'd')
-            toggleState(window, WinStateDockHorizontal);
-            */
+        else if (k == 'A') {
+            int sec = 5; tell("alarm %d seconds\n", sec); alarm(sec);
+        }
         else if (k == '0')
             setLayer(window, WinLayerDesktop);
         else if (k == '1')
@@ -537,7 +648,6 @@ int main(int argc, char **argv) {
             setLayer(window, WinLayerDock);
         else if (k == '5')
             setLayer(window, WinLayerAboveDock);
-#endif
         else if (k == 'm') {
             tell("%d %d\n", key.x_root, key.y_root);
             moveResize(window, key.x_root, key.y_root, 8); // move
@@ -545,6 +655,12 @@ int main(int argc, char **argv) {
             tell("%d %d\n", key.x_root, key.y_root);
             moveResize(window, key.x_root, key.y_root, 4); // _|
         }
+        else if (k == 'u') {
+            XMapRaised(display, unmapped);
+        }
+        else if (k == 'v') {
+            XMapRaised(display, unviewed);
+        }
         else if (k == 'x' || k == 'X' || k == CTRL('X')) {
             Window w = m == ShiftMask ? window :
                 m == ControlMask ? root : unmapped;
@@ -589,7 +705,9 @@ int main(int argc, char **argv) {
                 sscanf(buf, " %ld", &layout.rows);
             setLayout(root);
         }
-        else {
+        else if (k != XK_Shift_L && k != XK_Shift_R &&
+                 k != XK_Control_L && k != XK_Control_R)
+        {
             tell("Unrecognized key %d\n", k);
         }
     } break;
@@ -605,98 +723,26 @@ int main(int argc, char **argv) {
         unsigned char *prop(0);
 
         if (property.window == root &&
-            property.atom == _XA_WIN_WORKSPACE) {
-            if (XGetWindowProperty(display, root,
-                                   _XA_WIN_WORKSPACE_COUNT,
-                                   0, 1, False, AnyPropertyType,
-                                   &r_type, &r_format,
-                                   &count, &bytes_remain,
-                                   &prop) == Success && prop)
-            {
-                TEST(r_type == XA_CARDINAL && r_format == 32 && count == 1);
-                workspaceCount = ((long *)prop)[0];
-                XFree(prop);
-            }
-            if (XGetWindowProperty(display, root,
-                                   _XA_WIN_WORKSPACE,
-                                   0, 1, False, AnyPropertyType,
-                                   &r_type, &r_format,
-                                   &count, &bytes_remain,
-                                   &prop) == Success && prop)
-            {
-                TEST(r_type == XA_CARDINAL && r_format == 32 && count == 1);
-                activeWorkspace = ((long *)prop)[0];
-                tell("active=%ld of %ld\n", activeWorkspace, workspaceCount);
-                XFree(prop);
-            }
+            property.atom == _XA_NET_CURRENT_DESKTOP) {
+            updateWorkspaceCount();
+            updateActiveWorkspace();
+            tell("active=%ld of %ld\n", activeWorkspace, workspaceCount);
         }
         else if (property.window == root &&
-                 property.atom == _XA_WIN_WORKSPACE_NAMES) {
+                 property.atom == _XA_NET_DESKTOP_NAMES) {
         }
-#if 0
-        else if (property.atom == _XA_WIN_WORKAREA) {
-            if (XGetWindowProperty(display, root,
-                                   _XA_WIN_WORKAREA,
-                                   0, 4, False, AnyPropertyType,
-                                   &r_type, &r_format,
-                                   &count, &bytes_remain,
-                                   &prop) == Success && prop)
-            {
-                TEST(r_type == XA_CARDINAL && r_format == 32 && count == 4);
-                long *area = (long *)prop;
-                tell("workarea: min=%d,%d max=%d,%d\n",
-                       area[0],
-                       area[1],
-                       area[2],
-                       area[3]);
-                XFree(prop);
-            }
-        }
-#endif
         else if (property.window == window &&
-                 property.atom == _XA_WIN_WORKSPACE) {
-            if (XGetWindowProperty(display, root,
-                                   _XA_WIN_WORKSPACE_COUNT,
-                                   0, 1, False, AnyPropertyType,
-                                   &r_type, &r_format,
-                                   &count, &bytes_remain,
-                                   &prop) == Success && prop)
-            {
-                TEST(r_type == XA_CARDINAL && r_format == 32 && count == 1);
-                workspaceCount = ((long *)prop)[0];
-                XFree(prop);
-            }
-            if (XGetWindowProperty(display, window,
-                                   _XA_WIN_WORKSPACE,
-                                   0, 1, False, AnyPropertyType,
-                                   &r_type, &r_format,
-                                   &count, &bytes_remain,
-                                   &prop) == Success && prop)
-            {
-                TEST(r_type == XA_CARDINAL && r_format == 32 && count == 1);
-                windowWorkspace = ((long *)prop)[0];
-                tell("workspace %ld of %ld\n", windowWorkspace, workspaceCount);
-                XFree(prop);
-            }
+                 property.atom == _XA_NET_WM_DESKTOP) {
+            updateWindowWorkspace();
         }
-        else if (property.atom == _XA_WIN_STATE) {
-            long state[2] = {};
-            if (XGetWindowProperty(display, window,
-                                   _XA_WIN_STATE,
-                                   0, 2, False, AnyPropertyType,
-                                   &r_type, &r_format,
-                                   &count, &bytes_remain,
-                                   &prop) == Success && prop)
-            {
-                TEST(r_type == XA_CARDINAL && r_format == 32 && count == 2);
-                state[0] = ((long *)prop)[0];
-                state[1] = ((long *)prop)[1];
-                tell("win state %lX %lX\n", state[0], state[1]);
-                XFree(prop);
-            }
+        else if (property.window == root &&
+                 property.atom == _XA_NET_ACTIVE_WINDOW) {
+            long win = None;
+            getProperty(root, _XA_NET_ACTIVE_WINDOW, XA_WINDOW, &win);
+            tell("active window = 0x%lX\n", win);
         }
         else if (property.atom == _XA_NET_WM_STATE) {
-            if (XGetWindowProperty(display, window,
+            if (XGetWindowProperty(display, property.window,
                                    _XA_NET_WM_STATE,
                                    0, 20, False, AnyPropertyType,
                                    &r_type, &r_format,
@@ -704,17 +750,21 @@ int main(int argc, char **argv) {
                                    &prop) == Success && prop)
             {
                 TEST(r_type == XA_ATOM && r_format == 32);
-                tell("net wm state  ");
+                tell("net wm state %s: ",
+                        property.window == window ? "window" :
+                        property.window == unmapped ? "unmapped" :
+                        property.window == unviewed ? "unviewed" :
+                        "unknown");
                 for (unsigned long i = 0; i < count; ++i) {
                     printf("%s%s", i ? ", " : "",
                             atomName(((long *)prop)[i]));
                 }
-                printf("\n");
+                printf("%s\n", count == 0 ? "_" : "");
                 XFree(prop);
             }
         }
         else if (property.atom == _XA_WM_STATE) {
-            if (XGetWindowProperty(display, window,
+            if (XGetWindowProperty(display, property.window,
                                    _XA_WM_STATE,
                                    0, 2, False, AnyPropertyType,
                                    &r_type, &r_format,
@@ -722,16 +772,20 @@ int main(int argc, char **argv) {
                                    &prop) == Success && prop)
             {
                 TEST(r_type == _XA_WM_STATE && r_format == 32 && count == 2);
-                tell("wm state");
+                tell("wm state %s:",
+                        property.window == window ? "window" :
+                        property.window == unmapped ? "unmapped" :
+                        property.window == unviewed ? "unviewed" :
+                        "unknown");
                 for (unsigned i = 0; i < count; ++i) {
                     printf("%s%ld", i ? ", " : "  ", ((long *)prop)[i]);
                 }
-                printf("\n");
+                printf("%s\n", count == 0 ? "_" : "");
                 XFree(prop);
             }
         }
         else if (property.atom == _XA_WIN_LAYER) {
-            if (XGetWindowProperty(display, window,
+            if (XGetWindowProperty(display, property.window,
                                    _XA_WIN_LAYER,
                                    0, 1, False, AnyPropertyType,
                                    &r_type, &r_format,
@@ -740,12 +794,16 @@ int main(int argc, char **argv) {
             {
                 TEST(r_type == XA_CARDINAL && r_format == 32 && count == 1);
                 long layer = ((long *)prop)[0];
-                tell("win layer %ld\n", layer);
+                tell("win layer %s: %ld\n",
+                        property.window == window ? "window" :
+                        property.window == unmapped ? "unmapped" :
+                        property.window == unviewed ? "unviewed" :
+                        "unknown", layer);
                 XFree(prop);
             }
         }
         else if (property.atom == _XA_ICEWM_GUI_EVENT) {
-            if (XGetWindowProperty(display, window,
+            if (XGetWindowProperty(display, property.window,
                                    _XA_ICEWM_GUI_EVENT,
                                    0, 1, False, AnyPropertyType,
                                    &r_type, &r_format,
@@ -760,7 +818,7 @@ int main(int argc, char **argv) {
             }
         }
         else if (property.atom == _XA_WIN_TRAY) {
-            if (XGetWindowProperty(display, window,
+            if (XGetWindowProperty(display, property.window,
                                    _XA_WIN_TRAY,
                                    0, 1, False, AnyPropertyType,
                                    &r_type, &r_format,
@@ -777,6 +835,7 @@ int main(int argc, char **argv) {
             Window w = property.window;
             const char *s = w == window ? "window" :
                             w == unmapped ? "unmapped" :
+                            w == unviewed ? "unviewed" :
                             w == root ? "root" : "other!";
             Atom type(None);
             int format(0);
@@ -784,7 +843,7 @@ int main(int argc, char **argv) {
             unsigned long after(0);
             unsigned char* prop(0);
             if (XGetWindowProperty(display, w, atom,
-                                   0, 1000000, False, AnyPropertyType,
+                                   0, 4, False, AnyPropertyType,
                                    &type, &format, &nitems,
                                    &after, &prop) == Success && prop) {
                 tell("%-8s %-26s %2d %s\n", s, atomName(atom), format,
@@ -799,6 +858,26 @@ int main(int argc, char **argv) {
                     if (n)
                         printf("\n");
                 }
+                else if (atom == XA_WINDOW) {
+                    TEST(type == XA_WINDOW && format == 32 && nitems == 1);
+                    long* data = (long *) prop;
+                    int n = int(nitems);
+                    for (int i = 0; i < n; ++i) {
+                        printf("%s%ld", i ? ", " : "\t\t", data[i]);
+                    }
+                    if (n)
+                        printf("\n");
+                }
+                else if (atom == XA_CARDINAL) {
+                    TEST(type == XA_CARDINAL && format == 32 && nitems == 1);
+                    long* data = (long *) prop;
+                    int n = int(nitems);
+                    for (int i = 0; i < n; ++i) {
+                        printf("%s%ld", i ? ", " : "\t\t", data[i]);
+                    }
+                    if (n)
+                        printf("\n");
+                }
                 XFree(prop);
             }
             else {
@@ -809,8 +888,47 @@ int main(int argc, char **argv) {
     } break;
 
     default:
+        tell("%s\n", eventName(xev.type));
         break;
     }
 }
 
+int main(int argc, char **argv) {
+
+    signal(SIGTERM, sigcatch);
+    signal(SIGALRM, sigcatch);
+
+    bool pinging = false;
+
+    for (int opt; (opt = getopt(argc, argv, "d:epsh?")) > 0; ) {
+        switch (opt) {
+        case 'd':
+            setenv("DISPLAY", optarg, True);
+            break;
+
+        case 'e':
+            XSetErrorHandler(xfail);
+            break;
+
+        case 'p':
+            pinging ^= true;
+            break;
+
+        case 's':
+            XSynchronize(display, True);
+            break;
+
+        default:
+            help(*argv);
+        }
+    }
+    if (optind < argc) {
+        printf("%s: bad arg '%s'.\n", *argv, argv[optind]);
+        help(*argv);
+    } else {
+        test_run(*argv, pinging);
+    }
+    return 0;
+}
+
 // vim: set sw=4 ts=4 et:
