@@ -30,6 +30,7 @@
 #include "ascii.h"
 #include "ycursor.h"
 #include "yxcontext.h"
+#include "ytooltip.h"
 #ifdef CONFIG_XFREETYPE
 #include <ft2build.h>
 #include <X11/Xft/Xft.h>
@@ -64,6 +65,7 @@ YCursor YWMApp::scrollDownPointer;
 
 lazy<MoveMenu> moveMenu;
 lazy<TileMenu> tileMenu;
+lazy<TabsMenu> tabsMenu;
 lazy<LayerMenu> layerMenu;
 lazily<SharedWindowList> windowListMenu;
 lazy<LogoutMenu> logoutMenu;
@@ -350,6 +352,10 @@ CtrlAltDelete* YWMApp::getCtrlAltDelete() {
     return ctrlAltDelete;
 }
 
+AToolTip* YWMApp::newToolTip() {
+    return new YToolTip;
+}
+
 void YWMApp::subdirs(const char* subdir, bool themeOnly, MStringArray& paths) {
     if (resourcePaths.isEmpty()) {
         upath privDir(YApplication::getPrivConfDir());
@@ -557,6 +563,27 @@ void MoveMenu::updatePopup() {
     }
 }
 
+void TabsMenu::updatePopup() {
+    removeAll();
+    YFrameWindow* frame = dynamic_cast<YFrameWindow*>(getActionListener());
+    if (frame) {
+        for (YFrameClient* tab : frame->clients()) {
+            YAction action(EAction(int(tab->handle())));
+            YMenuItem* item = new YMenuItem(tab->windowTitle(), -1,
+                                            null, action, nullptr);
+            if (item) {
+                ref<YIcon> icon = tab->getIcon();
+                if (icon != null) {
+                    item->setIcon(icon);
+                }
+            }
+            add(item);
+        }
+        addSeparator();
+        addItem(_("Move to New _Window"),    -2, null, actionUntab);
+    }
+}
+
 void TileMenu::updatePopup() {
     if (itemCount()) {
         enableCommand(actionNull);
@@ -636,6 +663,8 @@ YMenu* YWMApp::getWindowMenu() {
 
     if (strchr(winMenuItems, 'i') && taskBarShowTray)
         windowMenu->addItem(_("Tray _icon"), -2, null, actionToggleTray);
+    if (strchr(winMenuItems, 'e'))
+        windowMenu->addItem(_("R_ename title"), -2, null, actionRename);
 
     if (strchr(winMenuItems, 'c') || strchr(winMenuItems, 'k'))
         windowMenu->addSeparator();
@@ -918,9 +947,7 @@ void YWMApp::actionPerformed(YAction action, unsigned int /*modifiers*/) {
             restartClient(nullptr, nullptr);
     }
     else if (action == actionRestartXterm) {
-        if (fRestartMsgBox) {
-            fRestartMsgBox->unmanage();
-        }
+        delete fRestartMsgBox;
         fRestartMsgBox = new YMsgBox(YMsgBox::mbBoth,
                                      _("Confirm Restart as Terminal"),
                                      _("Unmanage all applications and restart\n"
@@ -953,7 +980,7 @@ void YWMApp::actionPerformed(YAction action, unsigned int /*modifiers*/) {
     }
     else if (action == actionAboutClose) {
         if (aboutDlg) {
-            manager->unmanageClient(aboutDlg);
+            delete aboutDlg;
             aboutDlg = nullptr;
         }
     }
@@ -995,7 +1022,7 @@ void YWMApp::actionPerformed(YAction action, unsigned int /*modifiers*/) {
         keyProgs.clear();
         MenuLoader(this, this, this).loadMenus(findConfigFile("keys"), nullptr);
         if (manager && !initializing) {
-            if (manager->wmState() == YWindowManager::wmRUNNING) {
+            if (manager->isRunning()) {
                 manager->grabKeys();
             }
         }
@@ -1352,15 +1379,15 @@ YWMApp::YWMApp(int *argc, char ***argv, const char *displayName,
 
 YWMApp::~YWMApp() {
     if (fLogoutMsgBox) {
-        fLogoutMsgBox->unmanage();
+        delete fLogoutMsgBox;
         fLogoutMsgBox = nullptr;
     }
     if (fRestartMsgBox) {
-        fRestartMsgBox->unmanage();
+        delete fRestartMsgBox;
         fRestartMsgBox = nullptr;
     }
     if (aboutDlg) {
-        manager->unmanageClient(aboutDlg);
+        delete aboutDlg;
         aboutDlg = nullptr;
     }
 
@@ -1395,14 +1422,17 @@ YWMApp::~YWMApp() {
     layerMenu = null;
     moveMenu = null;
     tileMenu = null;
+    tabsMenu = null;
 
     keyProgs.clear();
     workspaces.reset();
     WPixRes::freePixmaps();
+    delete manager; manager = nullptr;
 
     extern void clearFontCache();
     clearFontCache();
 
+    configKeyboards.clear();
     YConfig::freeConfig(wmapp_preferences);
 
     XFlush(display());
@@ -1678,6 +1708,9 @@ static void print_configured(const char *argv0) {
 #ifdef LOGEVENTS
     " logevents"
 #endif
+#ifdef CONFIG_NANOSVG
+    " nanosvg"
+#endif
 #ifdef ENABLE_NLS
     " nls"
 #endif
@@ -1835,9 +1868,7 @@ void YWMApp::doLogout(RebootShutdown reboot) {
     if (!confirmLogout)
         logout();
     else {
-        if (fLogoutMsgBox) {
-            fLogoutMsgBox->unmanage();
-        }
+        delete fLogoutMsgBox;
         fLogoutMsgBox = new YMsgBox(YMsgBox::mbBoth,
                             _("Confirm Logout"),
                             _("Logout will close all active applications.\n"
@@ -1885,14 +1916,14 @@ void YWMApp::cancelLogout() {
 
 void YWMApp::handleMsgBox(YMsgBox *msgbox, int operation) {
     if (msgbox == fLogoutMsgBox) {
-        msgbox->unmanage();
+        delete fLogoutMsgBox;
         fLogoutMsgBox = nullptr;
         if (operation == YMsgBox::mbOK) {
             logout();
         }
     }
     if (msgbox == fRestartMsgBox) {
-        msgbox->unmanage();
+        delete fRestartMsgBox;
         fRestartMsgBox = nullptr;
         if (operation == YMsgBox::mbOK) {
             restartClient(TERM, nullptr);
