@@ -104,6 +104,8 @@ public:
     CtrlAltDelete* getCtrlAltDelete();
     const char* getConfigFile() const { return configFile; }
     FocusModel getFocusMode() const { return focusMode; }
+
+    AToolTip* newToolTip();
     YMenu* getWindowMenu();
     void subdirs(const char* subdir, bool themeOnly, MStringArray& paths);
     void unregisterProtocols();
@@ -203,6 +205,12 @@ public:
 };
 extern lazy<TileMenu> tileMenu;
 
+class TabsMenu : public SharedMenu {
+public:
+    void updatePopup();
+};
+extern lazy<TabsMenu> tabsMenu;
+
 class KProgram;
 typedef YObjectArray<KProgram> KProgramArrayType;
 typedef KProgramArrayType::IterType KProgramIterType;
