@@ -4,26 +4,21 @@
  * Copyright (C) 1997-2001 Marko Macek
  */
 #include "config.h"
-#include "ylib.h"
-#include <X11/keysym.h>
 #include "wmcontainer.h"
 #include "wmframe.h"
 #include "wmmgr.h"
-#include "yxapp.h"
+#include "wmapp.h"
 #include "prefs.h"
 
-#include <stdio.h>
-
-YClientContainer::YClientContainer(YWindow *parent, YFrameWindow *frame,
-                                   int depth, Visual *visual, Colormap cmap)
-    : YWindow(parent, None, depth, visual, cmap)
+YClientContainer::YClientContainer(YFrameWindow *frame, int depth,
+                                   Visual *visual, Colormap cmap)
+    : YWindow(frame, None, depth, visual, cmap)
+    , fFrame(frame)
+    , fHaveGrab(false)
+    , fHaveActionGrab(false)
 {
-    fFrame = frame;
-    fHaveGrab = false;
-    fHaveActionGrab = false;
-
     setStyle(wsManager | wsNoExpose);
-    setPointer(YXApplication::leftPointer);
+    setPointer(YWMApp::leftPointer);
     setTitle("Container");
     show();
 }
@@ -44,15 +39,14 @@ void YClientContainer::handleButton(const XButtonEvent &button) {
     {
         if (focusOnClickClient) {
             if (!getFrame()->isTypeDock()) {
-                doActivate = true;
+                doActivate = (getFrame() != manager->getFocus());
                 if (getFrame()->canFocusByMouse() && !getFrame()->focused())
                     firstClick = true;
             }
         }
-        if (raiseOnClickClient) {
+        if (raiseOnClickClient && getFrame()->canRaise()) {
             doRaise = true;
-            if (getFrame()->canRaise())
-                firstClick = true;
+            firstClick = true;
         }
     }
 
@@ -61,7 +55,7 @@ void YClientContainer::handleButton(const XButtonEvent &button) {
         unsigned int m = KEY_MODMASK(button.state);
         unsigned int vm = VMod(m);
 
-        if (IS_WMKEY(k, vm, gMouseWinSize)) {
+        if (gMouseWinSize.eq(k, vm)) {
             XAllowEvents(xapp->display(), AsyncPointer, CurrentTime);
 
             int px = button.x + x();
@@ -89,7 +83,8 @@ void YClientContainer::handleButton(const XButtonEvent &button) {
                                           mx, my);
             }
             return ;
-        } else if (IS_WMKEY(k, vm, gMouseWinMove)) {
+        }
+        else if (gMouseWinMove.eq(k, vm)) {
             XAllowEvents(xapp->display(), AsyncPointer, CurrentTime);
 
             if (getFrame()->canMove()) {
@@ -116,16 +111,16 @@ void YClientContainer::handleButton(const XButtonEvent &button) {
     }
 
     ///!!! do this first?
-    if (doActivate && getFrame() != manager->getFocus())
-        getFrame()->activate();
-    if (doRaise)
+    if (doActivate)
+        getFrame()->focus();
+    if (doRaise && (!doActivate || !raiseOnFocus))
         getFrame()->wmRaise();
     ///!!! it might be nice if this was per-window option (app-request)
     if (!firstClick || passFirstClickToClient)
         XAllowEvents(xapp->display(), ReplayPointer, CurrentTime);
     else
         XAllowEvents(xapp->display(), AsyncPointer, CurrentTime);
-    XSync(xapp->display(), False);
+    xapp->sync();
 }
 
 // manage button grab on frame window to capture clicks to client window
@@ -143,83 +138,64 @@ void YClientContainer::grabButtons() {
                        focusOnClickClient ||
                        raiseOnClickClient))
     {
+        for (int button = Button1; button <= Button3; ++button) {
+            if (buttonRaiseMask & (1 << (button - Button1))) {
+                XGrabButton(xapp->display(), button, AnyModifier,
+                            handle(), True, ButtonPressMask,
+                            GrabModeSync, GrabModeAsync, None, None);
+            }
+        }
         fHaveGrab = true;
-
-        XGrabButton(xapp->display(),
-                    Button1, AnyModifier,
-                    handle(), True,
-                    ButtonPressMask,
-                    GrabModeSync, GrabModeAsync, None, None);
-        XGrabButton(xapp->display(),
-                    Button2, AnyModifier,
-                    handle(), True,
-                    ButtonPressMask,
-                    GrabModeSync, GrabModeAsync, None, None);
-        XGrabButton(xapp->display(),
-                    Button3, AnyModifier,
-                    handle(), True,
-                    ButtonPressMask,
-                    GrabModeSync, GrabModeAsync, None, None);
     }
 }
 
 void YClientContainer::releaseButtons() {
     if (fHaveGrab) {
         fHaveGrab = false;
-
-        XUngrabButton(xapp->display(), Button1, AnyModifier, handle());
-        XUngrabButton(xapp->display(), Button2, AnyModifier, handle());
-        XUngrabButton(xapp->display(), Button3, AnyModifier, handle());
+        for (int button = Button1; button <= Button3; ++button) {
+            if (buttonRaiseMask & (1 << (button - Button1))) {
+                XUngrabButton(xapp->display(), button, AnyModifier, handle());
+            }
+        }
         fHaveActionGrab = false;
     }
     grabActions();
 }
 
 void YClientContainer::regrabMouse() {
-    XUngrabButton(xapp->display(), Button1, AnyModifier, handle());
-    XUngrabButton(xapp->display(), Button2, AnyModifier, handle());
-    XUngrabButton(xapp->display(), Button3, AnyModifier, handle());
-
-    if (fHaveActionGrab)  {
-        fHaveActionGrab = false;
-        grabActions();
-    }
-
-    if (fHaveGrab ) {
-        fHaveGrab = false;
-        grabButtons();
-    }
+    fHaveActionGrab = false;
+    releaseButtons();
+    grabButtons();
 }
 
 void YClientContainer::grabActions() {
     if (clientMouseActions && fHaveActionGrab == false) {
+        WMKey grab[] = { gMouseWinMove, gMouseWinSize,
+                         gMouseWinRaise, gMouseWinLower };
+        const int count = int ACOUNT(grab) - (gMouseWinRaise == gMouseWinLower);
+        for (int i = 0; i < count; ++i) {
+            int button = int(grab[i].key) - XK_Pointer_Button1 + Button1;
+            if (inrange(button, Button1, Button3)) {
+                grabVButton(button, grab[i].mod);
+            }
+        }
         fHaveActionGrab = true;
-        const KeySym minButton = XK_Pointer_Button1;
-        const KeySym maxButton = XK_Pointer_Button3;
-        const KeySym xkButton0 = XK_Pointer_Button1 - 1;
-        if (inrange(gMouseWinMove.key, minButton, maxButton))
-            grabVButton(gMouseWinMove.key - xkButton0, gMouseWinMove.mod);
-        if (inrange(gMouseWinSize.key, minButton, maxButton))
-            grabVButton(gMouseWinSize.key - xkButton0, gMouseWinSize.mod);
-        if (inrange(gMouseWinRaise.key, minButton, maxButton))
-            grabVButton(gMouseWinRaise.key - xkButton0, gMouseWinRaise.mod);
-        if (inrange(gMouseWinLower.key, minButton, maxButton))
-            grabVButton(gMouseWinLower.key - xkButton0, gMouseWinLower.mod);
     }
 }
 
 void YClientContainer::handleConfigureRequest(const XConfigureRequestEvent &configureRequest) {
     MSG(("configure request in frame"));
 
-    if (getFrame() &&
-        configureRequest.window == getFrame()->client()->handle())
+    if (configureRequest.window == getFrame()->client()->handle())
     {
         getFrame()->configureClient(configureRequest);
     }
 }
 
 void YClientContainer::handleMapRequest(const XMapRequestEvent &mapRequest) {
-    if (mapRequest.window == getFrame()->client()->handle()) {
+    if (mapRequest.window == getFrame()->client()->handle()
+        && getFrame()->isPassive() == false)
+    {
         if (getFrame()->isUnmapped()) {
             manager->lockFocus();
             getFrame()->makeMapped();
@@ -234,7 +210,7 @@ void YClientContainer::handleMapRequest(const XMapRequestEvent &mapRequest) {
 }
 
 void YClientContainer::handleCrossing(const XCrossingEvent &crossing) {
-    if (getFrame() && pointerColormap) {
+    if (pointerColormap) {
         if (crossing.type == EnterNotify)
             manager->setColormapWindow(getFrame());
         else if (crossing.type == LeaveNotify &&
@@ -247,5 +223,4 @@ void YClientContainer::handleCrossing(const XCrossingEvent &crossing) {
     }
 }
 
-
 // vim: set sw=4 ts=4 et:
