@@ -1,5 +1,5 @@
-#ifndef __WMDIALOG_H
-#define __WMDIALOG_H
+#ifndef WMDIALOG_H
+#define WMDIALOG_H
 
 #include "ywindow.h"
 #include "yaction.h"
@@ -31,7 +31,7 @@ private:
     YActionButton *aboutButton;
     YActionButton *windowListButton;
     IApp *app;
-    YActionButton *addButton(const ustring& str, unsigned& maxW, unsigned& maxH);
+    YActionButton *addButton(const mstring& str, unsigned& maxW, unsigned& maxH);
 };
 
 #endif
