@@ -29,6 +29,7 @@ public:
     virtual void handleDNDEnter();
     virtual void handleDNDLeave();
     void setGeometry();
+    int destination();
 private:
     YWindowManager *fManager;
     Cursor fCursor;
@@ -122,7 +123,8 @@ public:
     YFrameClient *findClient(Window win);
     void manageClient(YFrameClient* client, bool mapClient = false);
     void unmanageClient(YFrameClient *client);
-    void destroyedClient(Window win);
+    void clientDestroyed(YFrameClient* client);
+    void clientTransfered(YFrameClient* client, YFrameWindow* frame);
     void mapClient(Window win);
 
     void setFocus(YFrameWindow *f, bool canWarp = false, bool reorder = true);
@@ -172,14 +174,13 @@ public:
 
     void restackWindows();
     void focusTopWindow();
-    YFrameWindow *getFrameUnderMouse(long workspace = AllWorkspaces);
-    YFrameWindow *getLastFocus(bool skipAllWorkspaces = false, long workspace = AllWorkspaces);
+    YFrameWindow *getFrameUnderMouse(int workspace = AllWorkspaces);
+    YFrameWindow *getLastFocus(bool skipAllWorkspaces = false,
+                               int workspace = AllWorkspaces);
     void focusLastWindow();
     bool focusTop(YFrameWindow *f);
     void updateClientList();
     void updateUserTime(const UserTime& userTime);
-
-    int windowCount(long workspace);
     void popupWindowListMenu(YWindow *owner, int x, int y);
 
     void initWorkspaces();
@@ -198,16 +199,15 @@ public:
     void updateTaskBarNames();
     void updateMoveMenu();
 
-    bool readCurrentDesktop(long &workspace);
+    bool readCurrentDesktop(int &workspace);
     void setDesktopGeometry();
     bool compareDesktopNames(const YStringList& list);
     bool readDesktopLayout();
     void readDesktopNames(bool init, bool net);
     bool readNetDesktopNames(YStringList& list);
     bool readWinDesktopNames(YStringList& list);
-    void setWinDesktopNames(long count);
-    void setNetDesktopNames(long count);
-    void setDesktopNames(long count);
+    void setNetDesktopNames(int count);
+    void setDesktopNames(int count);
     void setDesktopNames();
     void setDesktopCount();
     void setDesktopViewport();
@@ -319,6 +319,7 @@ public:
     bool switchWindowVisible() const;
     SwitchWindow* getSwitchWindow();
     Window netActiveWindow() const { return fActiveWindow; }
+    int edgeWorkspace(int x, int y);
 
 private:
     struct WindowPosState {
@@ -328,6 +329,7 @@ private:
     };
 
     bool ignoreOverride(Window win, const XWindowAttributes& attr, int* layer);
+    bool tabbingClient(YFrameClient* client);
     YFrameClient* allocateClient(Window win, bool mapClient);
     YFrameWindow* allocateFrame(YFrameClient* client);
     void updateArea(int workspace, int screen_number, int l, int t, int r, int b);
