@@ -11,6 +11,7 @@ class ISwitchItems
 public:
     virtual void updateList() = 0;
     virtual int getCount() = 0;
+    virtual bool isEmpty() = 0;
     virtual ~ISwitchItems() {}
 
     // move the focused target up or down and return the new focused element
@@ -30,19 +31,24 @@ public:
     virtual int getActiveItem() = 0;
     virtual mstring getTitle(int idx) = 0;
     virtual ref<YIcon> getIcon(int idx) = 0;
+    virtual int getWorkspace(int idx) { return 0; }
 
     // Manager notification about windows disappearing under the fingers
-    virtual bool destroyedItem(YFrameWindow* frame) { return false; }
-    virtual bool createdItem(YFrameWindow* frame) { return false; }
+    virtual bool destroyedItem(YFrameWindow* frame, YFrameClient* client) {
+                    return false; }
+    virtual bool createdItem(YFrameWindow* frame, YFrameClient* client) {
+                    return false; }
+    virtual void transfer(YFrameClient* client, YFrameWindow* frame) { }
 
     virtual bool isKey(KeySym k, unsigned mod) = 0;
     virtual unsigned modifiers() = 0;
 
     // Filter items by WM_CLASS
-    virtual void setWMClass(char* wmclass) = 0;
+    virtual bool setWMClass(char* wmclass) = 0;
     virtual char* getWMClass() = 0;
 
     virtual YFrameWindow* current() const { return nullptr; }
+    virtual void sort() { }
 };
 
 class SwitchWindow: public YPopupWindow {
@@ -62,8 +68,11 @@ public:
     virtual bool handleKey(const XKeyEvent& key) override;
     virtual void handleButton(const XButtonEvent& button) override;
     virtual void handleMotion(const XMotionEvent& motion) override;
+    void destroyedClient(YFrameClient* client);
     void destroyedFrame(YFrameWindow* frame);
     void createdFrame(YFrameWindow* frame);
+    void createdClient(YFrameWindow* frame, YFrameClient* client);
+    void transfer(YFrameClient* client, YFrameWindow* frame);
     YFrameWindow* current();
 
 private:
@@ -76,6 +85,7 @@ private:
     // hints for fast identification of the entry under the cursor
     int m_hintAreaStart, m_hintAreaStep;
 
+    int fWorkspace;
     ref<YImage> fGradient;
 
     YColorName switchFg;
@@ -96,7 +106,7 @@ private:
     unsigned modifiers();
 
     void cancel();
-    bool close();
+    void close();
     void accept();
     void displayFocus();
     YFrameWindow* nextWindow(bool zdown);
