@@ -42,6 +42,7 @@ public:
 
     void append(const void *item);
     void insert(const SizeType index, const void *item);
+    void moveto(const SizeType index, const SizeType place);
     virtual void remove(const SizeType index);
     virtual void clear();
     virtual void shrink(const SizeType reducedCount);
@@ -711,15 +712,20 @@ YArrayIterator<DataType> YArray<DataType>::reverseIterator() {
 
 template<class DataType>
 int find(YArray<DataType>& array, DataType& data) {
-    for (YBaseArray::SizeType i = 0, n = array.getCount(); i < n; ++i)
-        if (array[i] == data) return i;
+    const YBaseArray::SizeType num = array.getCount();
+    if (num) {
+        for (DataType* ar = &*array, *pt = ar, *end = ar + num; pt < end; ++pt)
+            if (*pt == data)
+                return pt - ar;
+    }
     return -1;
 }
 
 template<class DataType>
 int find(const YArray<DataType>& array, const DataType& data) {
-    for (YBaseArray::SizeType i = 0, n = array.getCount(); i < n; ++i)
-        if (array[i] == data) return i;
+    for (const DataType& ref : array)
+        if (ref == data)
+            return &ref - array.begin();
     return -1;
 }
 
